@@ -52,6 +52,18 @@ DEoptim.control <- function(VTR = -Inf, strategy = 2, bs = FALSE, NP = NA,
     steptol <- itermax
   }
 
+  if(!(is.null(initialpop))) {
+    if(is.na(NP))
+      if(is.matrix(initialpop))
+        NP <- dim(initialpop)[1]
+      else stop("initialpop must be a matrix")
+    else
+      if(NP != dim(initialpop)[1]) {
+        warning("Resetting NP to the number of rows in initialpop")
+        NP <- dim(initialpop)[1]
+      }
+  }
+
   list(VTR = VTR, strategy = strategy, NP = NP, itermax = itermax, CR
        = CR, F = F, bs = bs, trace = trace, initialpop = initialpop,
        storepopfrom = storepopfrom, storepopfreq = storepopfreq, p =
@@ -110,37 +122,42 @@ DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...,
   ctrl$specinitialpop <- as.numeric(ctrl$specinitialpop)
   ctrl$initialpop <- as.numeric(ctrl$initialpop)
   if(ctrl$parallelType == 2) { ## use foreach 
-    use.foreach <- suppressMessages(require(foreach,quietly=TRUE))
-    if(!use.foreach)
-      stop("foreach package not available but parallelType set to 2")
-    if(!foreach:::getDoParRegistered()) {
-      foreach:::registerDoSEQ()
+    if(!foreach::getDoParRegistered()) {
+      foreach::registerDoSEQ()
     }
     args <-  ctrl$foreachArgs
     fnPop <- function(params, ...) {
-      my_chunksize <- ceiling(NROW(params)/foreach:::getDoParWorkers())
-      my_iter <- iter(params,by="row",chunksize=my_chunksize)
+      my_chunksize <- ceiling(NROW(params)/foreach::getDoParWorkers())
+      my_iter <- iterators::iter(params,by="row",chunksize=my_chunksize)
       args$i <- my_iter
       args$.combine <- c
-      args$.export <- "fn" 
-      do.call(foreach, args) %dopar% {
-        apply(i,1,fn,...)
-      }
+      if (!is.null(args$.export))
+        args$.export = c(args$.export, "fn")
+      else
+        args$.export = "fn"
+      if (is.null(args$.errorhandling))
+        args$.errorhandling = c('stop', 'remove', 'pass')
+      if (is.null(args$.verbose))
+        args$.verbose = FALSE
+      if (is.null(args$.inorder))
+        args$.inorder = TRUE
+      if (is.null(args$.multicombine))
+        args$.multicombine = FALSE
+      
+      foreach::"%dopar%"(do.call(foreach::foreach, args), apply(i,1,fn,...))
+      
     }
   }
   else if(ctrl$parallelType == 1){ ## use parallel 
-    use.parallel <- suppressMessages(require(parallel,quietly=TRUE))
-    if(!use.parallel)
-      stop("parallel package not available but parallelType set to 1")
-    cl <- parallel:::makeCluster(parallel:::detectCores())
+    cl <- parallel::makeCluster(parallel::detectCores())
     packFn <- function(packages) {
       for(i in packages)
         library(i, character.only = TRUE)
     }
-    parallel:::clusterCall(cl, packFn, ctrl$packages)
-    parallel:::clusterExport(cl, ctrl$parVar)
+    parallel::clusterCall(cl, packFn, ctrl$packages)
+    parallel::clusterExport(cl, ctrl$parVar)
     fnPop <- function(params, ...) {
-      parallel:::parApply(cl=cl,params,1,fn,...)
+      parallel::parApply(cl=cl,params,1,fn,...)
     }
   }
   else {  ## use regular for loop / apply
@@ -185,7 +202,7 @@ DEoptim <- function(fn, lower, upper, control = DEoptim.control(), ...,
   outC <- .Call("DEoptimC", lower, upper, fnPop, ctrl, new.env(), fnMapC, PACKAGE="DEoptim")
 
   if(ctrl$parallelType == 1)
-    parallel:::stopCluster(cl) 
+    parallel::stopCluster(cl) 
   
   if (length(outC$storepop) > 0) {
     nstorepop <- floor((outC$iter - ctrl$storepopfrom) / ctrl$storepopfreq)
